@@ -9,13 +9,13 @@ Number::Nary - encode and decode numbers as n-ary strings
 
 =head1 VERSION
 
-version 0.05
+version 0.100
 
- $Id$
+ $Id: /my/cs/projects/nary/trunk/lib/Number/Nary.pm 22024 2006-05-12T03:27:50.632712Z rjbs  $
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.100';
 
 use Carp qw(croak);
 use List::MoreUtils qw(uniq);
@@ -38,34 +38,62 @@ sub _generate_codec_pair {
 =head1 SYNOPSIS
 
 This module lets you convert numbers into strings that encode the number using
-the digit set of your choice.
+the digit set of your choice.  For example, you could get routines to convert
+to and from hex like so:
 
-  my ($encode_sub, $decode_sub) = n_codec('01234'); # base five
+  my ($enc_hex, $dec_hex) = n_codec('0123456789ABCDEF');
 
-  my $number = $encode_sub('8675309'); # jenny's number for one-handed folk
+  my $hex = $enc_hex(255);  # sets $hex to FF
+  my $num = $dec_hex('A0'); # sets $num to 160
+
+This would be slow and stupid, since Perl already provides the means to easily
+and quickly convert between decimal and hex representations of numbers.
+Number::Nary's utility comes from the fact that it can encode into bases
+composed of arbitrary digit sets.
+
+  my ($enc, $dec) = n_codec('0123'); # base 4 (for working with nybbles?)
+
+  # base64
+  my ($enc, $dec) = n_codec(
+    join('', 'A' .. 'Z', 'a' .. 'z', 0 .. 9, '+', '/', '=')
+  );
 
 =head1 FUNCTIONS
 
-=head2 C<< n_codec($digits) >>
+=head2 n_codec
+
+  my ($encode_sub, $decode_sub) = n_codec($digit_string);
 
 This routine returns a reference to a subroutine which will encode numbers into
 the given set of digits and a reference which will do the reverse operation.
 
-This routine will croak if the digit string contains repeated digits.
+The digits may be given as a string or an arrayref.  This routine will croak if
+the set of digits contains repeated digits, or if not all digits are of the
+same length of characters.
 
 The encode sub will croak if it is given input other than a non-negative
 integer. 
 
 The decode sub will croak if given a string that contains characters not in the
-digit string.
+digit string, or if the lenth of the string to decode is not a multiple of the
+length of the component digits.
 
 =cut
 
 sub n_codec {
 	my ($base_string) = @_;
 
-	my @digits = split //, $base_string;
-  croak "base string contains repeated characters"
+	my @digits;
+  my $length = 1;
+  if (eval { @digits = @$base_string; 1 }) {
+    my @lengths = uniq map { length } @digits;
+    croak "given digits were not of uniform length" unless @lengths == 1;
+    $length = length $digits[0];
+  } else {
+    @digits = split //, $base_string;
+  }
+
+  croak "digit list contains repeated characters"
     unless @digits == uniq @digits;
 
   my $encode_sub = sub {
@@ -73,8 +101,7 @@ sub n_codec {
 
     croak "value isn't an non-negative integer"
       if not defined $value
-      or $value !~ /\A\d+\z/
-      or ($value < 0);
+      or $value !~ /\A\d+\z/;
 
       my $string = '';
       while (1) {
@@ -86,20 +113,24 @@ sub n_codec {
     return $string;
   };
 
-  my $i = 0;
-  my %digit_value = map { $_ => $i++ } @digits;
+  my %digit_value = do { my $i = 0; map { $_ => $i++ } @digits; };
 
   my $decode_sub = sub {
     my ($string) = @_;
-    my @found_digits = split //, $string;
-    return 0 unless @found_digits;
+    return unless $string;
+
+    my $places = length($string) / $length;
+
+    croak "string length is not a multiple of digit length"
+      unless $places == int $places;
 
     my $value    = 0;
-    my $position = 0;
-    for my $digit (reverse @found_digits) {
+
+    for my $position (reverse 1 .. $places) {
+      my $digit = substr $string, (-$length * $position), $length;
       croak "string to decode contains invalid digits"
         unless exists $digit_value{$digit};
-      $value += $digit_value{$digit}  *  @digits ** $position++;
+      $value += $digit_value{$digit}  *  @digits ** ($position++ - 1);
     }
     return $value;
   };
@@ -107,18 +138,23 @@ sub n_codec {
   return ($encode_sub, $decode_sub);
 }
 
-=head2 C<< n_encode($value, $digits) >>
+=head2 n_encode
+
+  my $string = n_encode($value, $digit_string);
 
 This encodes the given value into a string using the given digit string.  It is
 written in terms of C<n_codec>, above, so it's not efficient at all for
 multiple uses in one process.
 
-=head2 C<< n_decode($value, $digits) >>
+=head2 n_decode
+
+  my $number = n_decode($string, $digit_string);
 
 This is the decoding equivalent to C<n_encode>, above.
 
 =cut
 
+# If you really can't stand using n_codec, you could memoize these.
 sub n_encode { (n_codec($_[1]))[0]->($_[0]) }
 sub n_decode { (n_codec($_[1]))[1]->($_[0]) }
 
@@ -165,7 +201,7 @@ a fun little distraction.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005 Ricardo Signes, all rights reserved.
+Copyright 2005-2006 Ricardo Signes, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
